// -*- c++ -*- for emacs
//
// Config -- Configuration parser and holder
//
// This object parses configuration files and creates a multi-layer hash
// that programs can query via methods.  It allows the setting and saving
// of values as well.  Configurations files are apache style
//
// <KeyType key>
//     key      information
// </KeyType>
//
// key   information
//
// example:
//
// Port  143
// <Domain urbanrage.com>
//       Database         /usr/db/database
//       <Server pop.urbanrage.com>
//              OS                    solaris-2.7
//              Architecture          sparc
//       </Server>
// </Domain>
//
//
// $Id: Config.h,v 1.2 1999/02/27 23:18:42 eric Exp $
//

#ifndef _common_Config_h_
#define _common_Config_h_

#include <string>
#include <map.h>
#include <iostream>
#include <fstream>

class ConfigItem;

typedef map<string, ConfigItem *> Conf_Map_t;

class ConfigItem {
public:
  
  ConfigItem() { __value = NULL; __map = NULL; }
  ConfigItem(void* iValue) { __value = iValue; __map = NULL; }
  ConfigItem(Conf_Map_t* iMap) { __value = NULL; __map = iMap; }
  ~ConfigItem() {};

  void* GetValue() { return __value; }
  void SetValue(void *iValue) { __value = iValue; __map = NULL; }
  
  Conf_Map_t* GetMap() { return __map; }
  void SetMap(Conf_Map_t* iMap) { __map = iMap; __value = NULL; }

  bool IsMap() { return (__map != NULL); }

private:
  void*       __value;
  Conf_Map_t* __map;
};

class Config {
public:
  
  Config();
  Config(const char* iFilename, bool iDebug = false);

  ~Config();

  void LoadFile(const char* iFilename);
  
  void* GetItem(char* iName ...);
  bool SetItem(void* iItem, char* iName ...);
  bool SetLevel(char* iName ...);
  void ResetLevel() { __current_map = &__root; }
  Conf_Map_t* GetLevel() { return __current_map; }

  void DebugDump(); // dumps current level, all if current is root

protected:
  int _isMap(Conf_Map_t* iMap, string iName); // 0 = value, 1 = ref, -1 = doesn't exist
  void _read_level(ifstream &iFile, Conf_Map_t* iLevel, string iLevelName);
  void _dump_level(Conf_Map_t *iLevel, string iTab);

private:
  char*        __filename;
  Conf_Map_t   __root;
  Conf_Map_t*  __current_map;
  unsigned int __line;
  ostream*     __debug_ptr;
  ofstream     __debug_null;
};

#endif
